var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { SearchQuery } from 'ozone-search-helper';
import { getDefaultClient } from 'ozone-default-client';
import { OzoneFormat } from 'ozone-config';
import { Request } from 'typescript-http-client';
import * as HLS from 'hls-parser';
import { OzoneVideoUrl } from 'ozone-media-url';
/**
 * ozone-api-edit-video
 *
 * ES6 module to save selected video chunk
 *
 *
 * # Usage
 *
 * ```javaScript
 * import {OzoneApiEditVideo} from 'ozone-api-edit-video'
 * const ozoneApiEditVideo =  new OzoneApiEditVideo();
 *
 * // TODO get originalVideo from ozone-api-item
 * // TODO get selectedChunks from ozone-video-player
 *
 * ozoneApiEditVideo
 *    .createSubVideo(originalVideo, selectedChunks)
 *    .then((video) => {
 *       console.log('new video created with id', video.id)
 *    });
 * ```
 *
 *
 */
export class OzoneApiEditVideo {
    constructor() {
        this._ozoneMediaUrlCollection = new Map();
        /***/
    }
    mediaUrlFactory(video) {
        return __awaiter(this, void 0, void 0, function* () {
            if (video.id && this._ozoneMediaUrlCollection.has(video.id)) {
                return this._ozoneMediaUrlCollection.get(video.id);
            }
            else {
                const ozoneMediaUrl = new OzoneVideoUrl(video, getDefaultClient());
                this._ozoneMediaUrlCollection.set(video.id, ozoneMediaUrl);
                return ozoneMediaUrl;
            }
        });
    }
    _createNewPlayListFile(originalVideo, chunksList) {
        return __awaiter(this, void 0, void 0, function* () {
            const chunksListFlatten = Array().concat.apply([], chunksList);
            const mediaUrl = yield this.mediaUrlFactory(originalVideo);
            const url = yield mediaUrl.getPreferredVideoUrl();
            const request = new Request(url)
                .setMethod('GET');
            request.responseType = 'text';
            const data = yield getDefaultClient().call(request);
            const playList = HLS.parse(data.toString());
            playList.segments = playList.segments.filter((segment) => {
                const chunkToKeep = chunksListFlatten.find((v) => v === segment.uri);
                return typeof (chunkToKeep) !== 'undefined';
            });
            for (let i = 1; i < chunksList.length; i++) {
                const firstChunkName = chunksList[i][0];
                const discontinuityChunk = playList.segments.find((segment) => {
                    return segment.uri === firstChunkName;
                });
                if (discontinuityChunk) {
                    discontinuityChunk.discontinuity = true;
                }
            }
            return HLS.stringify(playList);
        });
    }
    _savePlayList(playList) {
        return __awaiter(this, void 0, void 0, function* () {
            const blobClient = getDefaultClient().blobClient();
            return blobClient.create(playList);
        });
    }
    _createBlobFile(playListBlob) {
        return __awaiter(this, void 0, void 0, function* () {
            const blobFile = {
                blob: playListBlob.id,
                uti: 'unofficial.m3uu-playlist',
                type: 'file'
            };
            const fileItemClient = getDefaultClient().itemClient('file');
            return fileItemClient.save(blobFile);
        });
    }
    getVideoFile(originalVideo) {
        return __awaiter(this, void 0, void 0, function* () {
            if (originalVideo.derivedFiles) {
                const mediaUrl = yield this.mediaUrlFactory(originalVideo);
                const fileTypeIdentifier = yield mediaUrl.getPreferredVideoFormat();
                if (typeof fileTypeIdentifier !== 'string') {
                    throw new Error('No video files found');
                }
                const fileType = yield this.getFileType(fileTypeIdentifier);
                const query = new SearchQuery();
                query.termQuery('fileType', fileType.id)
                    .and.idsQuery(originalVideo.derivedFiles);
                const fileItemClient = getDefaultClient().itemClient('file');
                const searchResult = yield fileItemClient.search(query.searchRequest);
                if (searchResult && searchResult.results) {
                    const file = searchResult.results[0];
                    if (file) {
                        return file;
                    }
                }
                throw new Error('Unable to find original File');
            }
            else {
                throw new Error('originalVideo has no file');
            }
        });
    }
    getFileType(type) {
        return __awaiter(this, void 0, void 0, function* () {
            let identifier = type;
            if (OzoneFormat.type[type]) {
                identifier = OzoneFormat.type[type];
            }
            const fileTypeCache = yield getDefaultClient().fileTypeClient().getFileTypeCache();
            const file = fileTypeCache.findByIdentifier(identifier);
            if (file)
                return file;
            throw Error('identifier not found');
        });
    }
    _createFolder(playListFile, originalVideoFile, chunks, originaFileTypeIdentifier) {
        return __awaiter(this, void 0, void 0, function* () {
            if (originalVideoFile.subFiles) {
                const subFiles = originalVideoFile.subFiles;
                const newSubFile = { 'index.m3u8': playListFile.id };
                chunks.forEach((chunk) => {
                    if (subFiles[chunk]) {
                        newSubFile[chunk] = subFiles[chunk];
                    }
                });
                const originalFileType = (yield this.getFileType(originaFileTypeIdentifier));
                const newFolder = {
                    type: 'file',
                    uti: 'public.folder',
                    fileType: originalFileType.id,
                    subFiles: newSubFile
                };
                const fileApi = getDefaultClient().itemClient('file');
                return fileApi.save(newFolder);
            }
            throw new Error('Video file has no subFile');
        });
    }
    _duplicateVideo(originalVideo, newFolder) {
        return __awaiter(this, void 0, void 0, function* () {
            const newVideo = JSON.parse(JSON.stringify(originalVideo)); // deep copy
            const now = (new Date()).toISOString();
            delete newVideo.id;
            delete newVideo.derivedFiles;
            delete newVideo.version;
            newVideo.file = newFolder.id;
            newVideo.creationDate = now;
            newVideo.modificationDate = now;
            newVideo.previewDate = now;
            const videoApi = getDefaultClient().itemClient('video');
            return videoApi.save(newVideo);
        });
    }
    /**
     * Create a new ozone video from a subset of an original.
     * @param originalVideo
     * @param chunks
     */
    createSubVideo(originalVideo, chunks) {
        return __awaiter(this, void 0, void 0, function* () {
            // console.log('originalVideo', originalVideo)
            const chunksListFlatten = Array().concat.apply([], chunks);
            const playListData = yield this._createNewPlayListFile(originalVideo, chunks);
            const playListBlob = yield this._savePlayList(playListData);
            // console.log('playListBlob', playListBlob)
            // create new ozone file referencing  our playListBlob
            const playListFile = yield this._createBlobFile(playListBlob);
            // console.log('playListFile', playListFile)
            const OriginalVideoFile = yield this.getVideoFile(originalVideo);
            // console.log('OriginalVideoFile', OriginalVideoFile)
            const mediaUrl = yield this.mediaUrlFactory(originalVideo);
            const originaFileTypeIdentifier = yield mediaUrl.getPreferredVideoFormat();
            if (typeof originaFileTypeIdentifier !== 'string') {
                throw new Error('No video files found');
            }
            const newFolder = yield this._createFolder(playListFile, OriginalVideoFile, chunksListFlatten, originaFileTypeIdentifier);
            // console.log('newFolder', newFolder)
            const newVideo = yield this._duplicateVideo(originalVideo, newFolder);
            // console.log(newVideo)
            return newVideo;
        });
    }
}
